@@ -17,7 +17,7 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define PROGRAM_VERSION "0.2.0"
+#define PROGRAM_VERSION "0.2.1"
 
 #include <arpa/inet.h>
 //#include <sys/types.h>
@@ -203,14 +203,46 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 
 		lease * a = get_lease_by_client_port(client, answer_packet.mapping.private_port);
 		if (a != NULL) {
-			/* lease exists, update expiration time of the requested protocol and answer with public port */
-			uint32_t new_expires = now + ntohl(answer_packet.mapping.lifetime);
-			if (new_expires == next_lease_expires);
-			else if (new_expires < next_lease_expires) next_lease_expires = new_expires;
-			else if (a->expires[(int) protocol] <= next_lease_expires) update_expires = 1;
-			a->expires[(int) protocol] = new_expires;
-			answer_packet.mapping.public_port = a->public_port;
-			if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s updated\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+			if (a->expires[(int) protocol] != UINT32_MAX) {
+				/* lease exists, update expiration time of the requested protocol and answer with public port */
+				uint32_t new_expires = now + ntohl(answer_packet.mapping.lifetime);
+				if (new_expires == next_lease_expires);
+				else if (new_expires < next_lease_expires) next_lease_expires = new_expires;
+				else if (a->expires[(int) protocol] <= next_lease_expires) update_expires = 1;
+				a->expires[(int) protocol] = new_expires;
+				answer_packet.mapping.public_port = a->public_port;
+				if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s updated\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+			}
+			else {
+				/* lease for other protocol exists, but not for this one*/
+				/* check for manual mapping */
+				/* TODO: remove redundant code */
+				uint16_t public_port;
+				int b = get_dnat_rule_by_client_port(protocol, &public_port, client, answer_packet.mapping.private_port);
+				if (b == -1) die("get_dnat_rule_by_client_port returned with error");
+				else if (b == 1) {
+					/* manual mapping exists, answer with public port */
+					answer_packet.mapping.public_port = public_port;
+					if (debuglevel >= 2) printf("Manual mapping for public %s port %hu for client %s exists\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+				}
+				else {
+					/* add the lease to the other one */
+					answer_packet.mapping.public_port = a->public_port;
+					/* add the lease to the database */
+					a->expires[(int) protocol] = now + ntohl(answer_packet.mapping.lifetime);
+					if (a->expires[(int) protocol] < next_lease_expires) next_lease_expires = a->expires[(int) protocol];
+					/* create the mapping*/
+					{
+						int c = create_dnat_rule(
+								protocol,
+								answer_packet.mapping.public_port,
+								client,
+								answer_packet.mapping.private_port);
+						if (c == -1) die("create_dnat_rule returned with error");
+					}
+					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s added\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+				}
+			}
 		}
 		else {
 			/* no lease exists, check for manual mapping */
@@ -337,7 +369,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 				else if (b == 1) {
 					/* manual mapping found, answer with refused */
 					answer_packet.answer.result = NATPMP_REFUSED;
-					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s is mapped manually\n", proto(protocol), answer_packet.mapping.public_port, inet_ntoa(t_addr->sin_addr));
+					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s is mapped manually\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 				}
 			}
 		}
@@ -350,6 +382,10 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 
 	/* fire the packet to the client */
 	send_natpmp_packet(ufd, t_addr, (natpmp_packet_answer *) &answer_packet, sizeof(answer_packet));
+
+#ifdef DEBUG_LEASES
+	if (debuglevel >= 2) print_leases();
+#endif
 }
 
 /* being called on unsupported requests */
@@ -766,7 +802,7 @@ int main(int argc, char * argv[]) {
 						if (b == -1) die("destroy_dnat_rule returned with error");
 						if (debuglevel >= 2) {
 							struct in_addr client = { a->client };
-							printf("Lease with public %s port %hu for client %s expired and removed\n", proto(protocol), a->public_port, inet_ntoa(client));
+							printf("Lease with public %s port %hu for client %s expired and removed\n", proto(protocol), ntohs(a->public_port), inet_ntoa(client));
 						}
 					}
 				}
@@ -778,6 +814,9 @@ int main(int argc, char * argv[]) {
 					/* lease is no more used, remove it */
 					remove_lease_by_pointer(a);
 				}
+#ifdef DEBUG_LEASES
+				if (debuglevel >= 2) print_leases();
+#endif
 			}
 		}
 
